# Tai Sakuma <tai.sakuma@gmail.com>
from alphatwirl.misc.removal import _removed

##__________________________________________________________________||
@_removed(msg='use alphatwirl.parallel.build.build_parallel() instead.')
def build_progressMonitor_communicationChannel(quiet, processes):
    pass

##__________________________________________________________________||
